@@ -1,6 +1,7 @@
 import json
 from datetime import datetime
 
+from instapv.response.self_user_feed import SelfUserFeedResponse
 
 class Account:
 
@@ -28,7 +29,7 @@ class Account:
 
     def get_current_user(self):
         query = self.bot.request('accounts/current_user/?edit=true')
-        return query
+        return SelfUserFeedResponse(query)
 
     def set_gender(self, biography: str):
         if not isinstance(biography, str) or len(biography) > 150:
